//=============================================================================
// IG_ProjectileBlast.
//=============================================================================
class IG_ProjectileBlast expands Projectile;

#EXEC MESH IMPORT MESH=IG_BlastMesh ANIVFILE=Models\IG_BlastMesh_a.3d DATAFILE=Models\IG_BlastMesh_d.3d
#EXEC MESH ORIGIN MESH=IG_BlastMesh X=0 Y=1100 Z=-20 YAW=64
#EXEC MESH SEQUENCE MESH=IG_BlastMesh SEQ=All STARTFRAME=0 NUMFRAMES=25
#EXEC MESH SEQUENCE MESH=IG_BlastMesh SEQ=Burst STARTFRAME=0 NUMFRAMES=25
#EXEC MESHMAP SCALE MESHMAP=IG_BlastMesh X=0.04 Y=0.04 Z=0.08
#EXEC MESHMAP SETTEXTURE MESHMAP=IG_BlastMesh NUM=1 TEXTURE=Damagewet

var float MassThreshold;
var bool bInit;

simulated function PostBeginPlay()
{
	PlayAnim('Burst',1.5);

	if ( Role != ROLE_Authority )
	return;

	if ( Instigator != None )
	MakeNoise(1.0);

	BlastActors();
}

simulated function Tick(float DeltaTime)
{
	if ( !bInit )
	{
		bInit = True;
		return;
	}

	Disable('Tick');

	if ( Level.NetMode != NM_DedicatedServer )
	Spawn(class'IG_BlastEffect');
}

function BlastActors()
{
	local Actor A;
	local vector HitLocation, HitNormal;
	local float MassFactor;

	if ( Instigator != None )
	A = Instigator.TraceShot(HitLocation,HitNormal,Location+Vector(Rotation)*90,Location);
	else
	A = Trace(HitLocation,HitNormal,Location+Vector(Rotation)*90,Location,True);

	if ( (Pawn(A) != None) || (Decoration(A) != None) || ((Mover(A) != None) && Mover(A).bDamageTriggered) || (Flare(A) != None) )
	{
		A.TakeDamage(Damage,Instigator,HitLocation,vect(0,0,0),'');

		if ( (Mover(A) != None) || (Flare(A) != None) )
		return;

		if ( (A.Physics == PHYS_Walking) || (A.Physics == PHYS_None) || (A.Physics == PHYS_Rotating) || (A.Physics == PHYS_Spider) )
		A.SetPhysics(PHYS_Falling);

		if ( A.Physics != PHYS_Falling )
		return;

		MassFactor = MassThreshold/FMax(A.Mass,MassThreshold);

		A.Velocity = MomentumTransfer*Vector(Rotation)*MassFactor;
		A.Velocity.Z = MomentumTransfer/3*MassFactor;
	}
}

simulated function AnimEnd()
{
	Destroy();
}

simulated function PreBeginPlay()
{
	Super.PreBeginPlay();

	if ( (Int(Level.EngineVersion) < 226) && (Level.NetMode != NM_DedicatedServer) )
	MultiSkins[1] = Texture'UDamageFX';
}

defaultproperties
{
     MassThreshold=250.000000
     Damage=0.000000
     MomentumTransfer=700
     LifeSpan=2.000000
     Mesh=Mesh'IG_BlastMesh'
     bUnlit=True
     AmbientGlow=255
     bCollideActors=False
     RemoteRole=ROLE_SimulatedProxy
}
