//=============================================================================
// IG_ProjectileDart.
//=============================================================================
class IG_ProjectileDart expands Projectile;

#EXEC MESH IMPORT MESH=IG_DartMesh ANIVFILE=Models\IG_DartMesh_a.3d DATAFILE=Models\IG_DartMesh_d.3d
#EXEC MESH ORIGIN MESH=IG_DartMesh X=0 Y=0 Z=0 PITCH=-64
#EXEC MESH SEQUENCE MESH=IG_DartMesh SEQ=All STARTFRAME=0 NUMFRAMES=1
#EXEC MESHMAP SCALE MESHMAP=IG_DartMesh X=0.01 Y=0.01 Z=0.02
#EXEC MESHMAP SETTEXTURE MESHMAP=IG_DartMesh NUM=1 TEXTURE=Damagewet

#EXEC AUDIO IMPORT NAME=IG_DartSound FILE=Sounds\IG_DartSound.wav
#EXEC AUDIO IMPORT NAME=IG_SplashSound FILE=Sounds\IG_SplashSound.wav

var bool bWindow, bKillOnRelease;
var name ImplantEvent, ReleaseEvent;

var IG_DartTrailer Trailer;

simulated function PostBeginPlay()
{
	if ( Level.NetMode != NM_DedicatedServer )
	Trailer = Spawn(class'IG_DartTrailer',Self);

	if ( Role != ROLE_Authority )
	return;

	Velocity = Vector(Rotation)*Speed;

	if ( Region.Zone.bWaterZone )
	Velocity *= 0.7;
}

simulated function ZoneChange(ZoneInfo NewZone)
{
	if ( NewZone.bWaterZone && !Region.Zone.bWaterZone )
	Velocity *= 0.7;
}

simulated function ProcessTouch(Actor Other, vector HitLocation)
{
	if ( Other == Instigator )
	return;

	if ( (Level.NetMode != NM_DedicatedServer) && !Region.Zone.bWaterZone && (Pawn(Other) == None) && (IG_Handler(Other) == None) )
	SpawnDrops(Rotator(HitLocation-Other.Location));

	Destroy();

	if ( (Role != ROLE_Authority) || (PlayerPawn(Instigator) == None) || Instigator.bDeleteMe )
	return;

	if ( IG_Handler(Other) != None )
	Instigator.ClientMessage("You cannot control other players.",'Pickup');

	if ( ((Pawn(Other) != None) && !SpawnImplantHandler(Pawn(Other))) || (IG_Handler(Other) != None) )
	Other.TakeDamage(Damage,Instigator,HitLocation,vect(0,0,0),MyDamageType);
}

simulated function HitWall(vector HitNormal, Actor Wall)
{
	if ( (BlockAll(Wall) != None) && (String(Wall.Tag) ~= "DartWindow") )
	{
		bWindow = True;

		Disable('Touch');

		Enable('Tick');

		return;
	}

	if ( (Role == ROLE_Authority) && (Instigator != None) )
	MakeNoise(0.3);

	if ( (Level.NetMode != NM_DedicatedServer) && !Region.Zone.bWaterZone )
	SpawnDrops(Rotator(HitNormal));

	Destroy();
}

simulated function Tick(float DeltaTime)
{
	if ( !bWindow )
	{
		Enable('Touch');

		Disable('Tick');
	}
	else
		bWindow = False;
}

simulated function SpawnDrops(rotator DropRot)
{
	local int i;

	for ( i = 5+Rand(3); i > 0; i-- )
	Spawn(class'IG_Drop',,,,DropRot);

	PlaySound(Sound'IG_SplashSound',SLOT_Interact,,,600,0.9+FRand()*0.1);
}

function bool SpawnImplantHandler(Pawn P)
{
	local IG_Handler Handler;
	local rotator SpawnRot;
	local float Duration;

	ForEach AllActors( class'IG_Handler', Handler )
	if ( Handler.ImplantPawn == P )
	{
		if ( Handler.Controller != Instigator )
		Instigator.ClientMessage("That pawn is already controlled by"@Handler.Controller.PlayerReplicationInfo.PlayerName$".",'Pickup');
		return False;
	}
	else if ( Handler.Controller == Instigator )
	{
		Instigator.ClientMessage("Only one pawn can be controlled at a time.",'Pickup');
		return False;
	}

	if ( ImplantGun(Instigator.Weapon) == None )
	{
		Instigator.ClientMessage("You cannot control a pawn without a selected ImplantGun.",'Pickup');
		return False;
	}

	if ( Instigator.IsInState('CheatFlying') )
	{
		Instigator.ClientMessage("You cannot control a pawn while flying/ghosting.",'Pickup');
		return False;
	}

	if ( Instigator.Region.Zone.bWaterZone )
	{
		Instigator.ClientMessage("You cannot control a pawn while underwater.",'Pickup');
		return False;
	}

	if ( !ValidatePlayerLocation() )
	{
		Instigator.ClientMessage("You cannot control a pawn from that location.",'Pickup');
		return False;
	}

	if ( PlayerPawn(P) != None )
	{
		Instigator.ClientMessage("You cannot control other players.",'Pickup');
		return False;
	}

	Duration = FindImplantInfo(P);

	if ( Duration <= 0 )
	{
		Instigator.ClientMessage("That pawn cannot be controlled.",'Pickup');
		return False;
	}

	SpawnRot.Yaw = Instigator.Rotation.Yaw;

	if ( SkaarjWarrior(P) != None )
	Handler = Spawn(class'IG_SkaarjWarrior',Instigator,,Instigator.Location,SpawnRot);
	else if ( (Krall(P) != None) && (LeglessKrall(P) == None) )
	Handler = Spawn(class'IG_Krall',Instigator,,Instigator.Location,SpawnRot);
	else if ( Brute(P) != None )
	Handler = Spawn(class'IG_Brute',Instigator,,Instigator.Location,SpawnRot);
	else if ( (Nali(P) != None) && (Caps(String(P.Class)) != "NALIWARRIOR.NALIWARRIOR") )
	Handler = Spawn(class'IG_Nali',Instigator,,Instigator.Location,SpawnRot);
	else if ( Mercenary(P) != None )
	Handler = Spawn(class'IG_Mercenary',Instigator,,Instigator.Location,SpawnRot);
	else if ( Titan(P) != None )
	Handler = Spawn(class'IG_Titan',Instigator,,Instigator.Location,SpawnRot);
	else if ( Gasbag(P) != None )
	Handler = Spawn(class'IG_Gasbag',Instigator,,Instigator.Location,SpawnRot);
	else if ( SkaarjTrooper(P) != None )
	Handler = Spawn(class'IG_SkaarjTrooper',Instigator,,Instigator.Location,SpawnRot);
	else if ( (Caps(String(P.Class)) == "PSXMINOTAUR.MINOTAUR") || (Caps(String(P.Class)) == "PSXMINOTAUR.BIGMINOTAUR") )
	Handler = Spawn(class'IG_Minotaur',Instigator,,Instigator.Location,SpawnRot);
	else if ( (Caps(String(P.Class)) == "UPSXDRAGON.DRAGON") || (Caps(String(P.Class)) == "UPSXDRAGON.FANG") )
	Handler = Spawn(class'IG_Dragon',Instigator,,Instigator.Location,SpawnRot);
	else if ( Fly(P) != None )
	Handler = Spawn(class'IG_Fly',Instigator,,Instigator.Location,SpawnRot);
	else if ( (Slith(P) != None) && (Caps(String(P.Class)) != "LAMPREY.LAMPREY") )
	Handler = Spawn(class'IG_Slith',Instigator,,Instigator.Location,SpawnRot);
	else if ( Queen(P) != None )
	Handler = Spawn(class'IG_Queen',Instigator,,Instigator.Location,SpawnRot);

	if ( Handler == None )
	{
		Instigator.ClientMessage("That pawn cannot be controlled.",'Pickup');
		return False;
	}

	Handler.Duration = Duration;
	Handler.Controller = PlayerPawn(Instigator);
	Handler.ImplantPawn = ScriptedPawn(P);
	Handler.Gun = ImplantGun(Instigator.Weapon);
	Handler.bKillOnRelease = bKillOnRelease;
	Handler.ImplantEvent = ImplantEvent;
	Handler.ReleaseEvent = ReleaseEvent;

	ImplantGun(Instigator.Weapon).Handler = Handler;
	Instigator.Weapon.bCanThrow = False;

	Instigator.ClientMessage("You control"@P.MenuName$".",'Pickup');

	return True;
}

function bool ValidatePlayerLocation()
{
	local SpecialEvent SE;
	local vector HitLocation, HitNormal;

	if ( (Instigator.TraceShot(HitLocation,HitNormal,Instigator.Location-vect(0,0,256),Instigator.Location) != Level) || ((Instigator.Base != None) && (Instigator.Base != Level)) )
	return False;

	SE = Spawn(class'SpecialEvent',,,HitLocation+vect(0,0,1)*Instigator.CollisionHeight);

	if ( SE != None )
	SE.Destroy();

	if ( (SE == None) || SE.Region.Zone.bWaterZone || SE.Region.Zone.bPainZone )
	return False;

	return True;
}

function float FindImplantInfo(Pawn P)
{
	local ImplantInfo Info;
	local bool bFound, bAllowOther;

	ForEach AllActors( class'ImplantInfo', Info )
	{
		bFound = True;

		if ( (Info.Tag != '') && (Info.Tag == P.Tag) )
		{
			bKillOnRelease = Info.bKillOnRelease;

			ImplantEvent = Info.ImplantEvent;

			ReleaseEvent = Info.ReleaseEvent;

			if ( Info.bTriggerOnceOnly )
			{
				Info.ImplantEvent = '';

				Info.ReleaseEvent = '';
			}

			return Info.ImplantDuration;
		}
		else if ( Info.bAllowOther )
			bAllowOther = True;
	}

	if ( bFound && !bAllowOther )
	return 0;

	return class'IG_Handler'.default.Duration;
}

simulated function Destroyed()
{
	if ( Trailer != None )
	Trailer.Destroy();
}

simulated function PreBeginPlay()
{
	Super.PreBeginPlay();

	if ( (Int(Level.EngineVersion) < 226) && (Level.NetMode != NM_DedicatedServer) )
	MultiSkins[1] = Texture'UDamageFX';
}

defaultproperties
{
     LifeSpan=8.000000
     Speed=1800.000000
     Damage=5.000000
     MyDamageType=jolted
     Mesh=Mesh'IG_DartMesh'
     Style=STY_Translucent
     bUnlit=True
     AmbientGlow=255
     LightType=LT_Steady
     LightEffect=LE_NonIncidence
     LightRadius=6
     LightBrightness=64
     LightHue=180
     LightSaturation=64
     bBounce=True
     bFixedRotationDir=True
     AmbientSound=Sound'IG_DartSound'
     SoundRadius=8
     SoundVolume=128
     RotationRate=(Pitch=0,Yaw=0,Roll=30000)
     RemoteRole=ROLE_SimulatedProxy
}
