//=============================================================================
// ImplantRecharger.
//=============================================================================
class ImplantRecharger expands Decoration;

#EXEC TEXTURE IMPORT NAME=IG_RechargerSkin FILE=Textures\IG_RechargerSkin.pcx LODSET=2 DETAIL=metal

#EXEC MESH IMPORT MESH=IG_RechargerMesh ANIVFILE=Models\IG_RechargerMesh_a.3d DATAFILE=Models\IG_RechargerMesh_d.3d
#EXEC MESH ORIGIN MESH=IG_RechargerMesh X=0 Y=0 Z=-200
#EXEC MESH LODPARAMS MESH=IG_RechargerMesh STRENGTH=0.1
#EXEC MESH SEQUENCE MESH=IG_RechargerMesh SEQ=All STARTFRAME=0 NUMFRAMES=1
#EXEC MESHMAP SCALE MESHMAP=IG_RechargerMesh X=0.025 Y=0.025 Z=0.05
#EXEC MESHMAP SETTEXTURE MESHMAP=IG_RechargerMesh NUM=0 TEXTURE=Damagewet
#EXEC MESHMAP SETTEXTURE MESHMAP=IG_RechargerMesh NUM=1 TEXTURE=IG_RechargerSkin

#EXEC AUDIO IMPORT NAME=IG_RechargeSound FILE=Sounds\IG_RechargeSound.wav
#EXEC AUDIO IMPORT NAME=IG_ErrorSound FILE=Sounds\IG_ErrorSound.wav

var() int MaxAmmo;
var() float AmmoPerSecond;

var PlayerPawn Player;

var bool bError;
var float LastError;

function Touch(Actor Other)
{
	if ( (PlayerPawn(Other) == None) || (Player != None) || (Pawn(Other).FindInventoryType(class'ImplantGun') == None) )
	return;

	Player = PlayerPawn(Other);

	bError = False;
}

function Tick(float DeltaTime)
{
	local PlayerPawn PP;
	local ImplantGun IG;

	if ( Player == None )
	return;

	ForEach TouchingActors( class'PlayerPawn', PP )
	if ( PP == Player )
	break;

	IG = ImplantGun(Player.FindInventoryType(class'ImplantGun'));

	if ( (PP == None) || Player.bDeleteMe || (Player.Health < 1) || (IG == None) )
	{
		Player = None;

		AmbientSound = None;

		return;
	}

	if ( (IG.AmmoType == None) || (IG.AmmoType.AmmoAmount >= MaxAmmo) || (IG.AmmoType.AmmoAmount >= IG.AmmoType.MaxAmmo) )
	{
		AmbientSound = None;

		if ( !bError && (Level.TimeSeconds-LastError > 0.5) )
		{
			bError = True;

			LastError = Level.TimeSeconds;

			PlaySound(Sound'IG_ErrorSound',,,,600);
		}

		IG.FlareOffset = 0;

		return;
	}

	if ( AmbientSound == None )
	AmbientSound = Sound'IG_RechargeSound';

	bError = False;

	IG.FlareOffset += DeltaTime;

	if ( IG.FlareOffset < 1/AmmoPerSecond )
	return;

	IG.FlareOffset = 0;

	IG.AmmoType.AmmoAmount++;
}

simulated function PreBeginPlay()
{
	Super.PreBeginPlay();

	if ( (Int(Level.EngineVersion) < 226) && (Level.NetMode != NM_DedicatedServer) )
	MultiSkins[0] = Texture'UDamageFX';
}

defaultproperties
{
     MaxAmmo=10
     AmmoPerSecond=0.500000
     bStatic=False
     DrawType=DT_Mesh
     Mesh=Mesh'IG_RechargerMesh'
     AmbientGlow=64
     CollisionRadius=16.000000
     CollisionHeight=8.000000
     bCollideActors=True
     SoundRadius=16
}
